/**
 * @file gpio_driver.h
 * 
 * @brief The GPIO driver
 * 
 * Provides functions to initialize the GPIOs and to set/get their values
 * 			
 * @author Tecnologix, 2014
 */
#ifndef GPIO_H
#define GPIO_H

#include <stm32f4xx.h>
#include <stdbool.h>

/* ======================================================================
 MACROS
 ======================================================================== */

#define TLX_GPIO_MODE_INPUT				(0x00ul)
#define TLX_GPIO_MODE_GPOUT				(0x01ul)
#define TLX_GPIO_MODE_AF					(0x02ul)
#define TLX_GPIO_MODE_ANALOG			(0x03ul)

#define TLX_GPIO_SPEED_2MHZ				(0x00ul)
#define TLX_GPIO_SPEED_25MHZ			(0x01ul)
#define TLX_GPIO_SPEED_50MHZ			(0x02ul)
#define TLX_GPIO_SPEED_100MHZ			(0x03ul)

/* ======================================================================
 FUNCTION PROTOTYPES
 ======================================================================== */

/**
 * Configure a microcontroller pin.
 * @param gpio A pointer to GPIO description structure.
 * @param pin_number The pin number.
 * @param mode Pin mode from @link HAL_GPIO_DEF_MODE Gpio mode configuration @endlink.
 * @param speed Pin speed from @link HAL_GPIO_DEF_SPEED Gpio speed configuration @endlink
 * @param alternate_function Pin alternate function selection.
 */
void gpio_set_pin(GPIO_TypeDef *port, uint32_t pin_number, uint32_t mode, uint32_t speed, uint32_t alternate_function);

/**
 * If a pin is configured in input with pull-up / pull-down mode, this function enable pull-up.
 * @param gpio A pointer to GPIO description structure.
 * @param pin_number The pin number.
 */
void gpio_enable_pullup(GPIO_TypeDef* gpio, uint8_t pin_number);

/**
 * Set a microcontroller gpio pin.
 * @param gpio A pointer to GPIO description structure.
 * @param pin_number The pin number.
 */
void gpio_set(GPIO_TypeDef* gpio, uint8_t pin_number);

/**
 * Clear a microcontroller gpio pin
 * @param gpio A pointer to GPIO description structure.
 * @param pin_number The pin number.
 */
void gpio_clr(GPIO_TypeDef* gpio, uint8_t pin_number);

/**
 * Get gpio status.
 * @param gpio A pointer to GPIO description structure.
 * @param pin_number The pin number.
 * @return true if pin is set to high voltage level, false otherwise.
 */
bool gpio_get_status(GPIO_TypeDef * gpio, uint8_t pin_number);

#endif
