/**
 * @file gpio_driver.c
 * 
 * Implementation of function bodies defined in gpio_driver.h
 * 
 * @author Tecnologix 2014
 */
#include "gpio.h"
#include <stm32f4xx.h>
#include <stdbool.h>

/* =============================================================================
 * PUBLIC FUNCTIONS 
 * ============================================================================= */

/*
 * gpio_set_pin
 * See header file for details
 **/
void gpio_set_pin(GPIO_TypeDef *port, uint32_t pin_number, uint32_t mode, uint32_t speed, uint32_t alternate_function)
{
	port->MODER &= ~(0x03ul << (pin_number << 1));
	port->MODER |= (mode << (pin_number << 1));

	port->OSPEEDR &= ~(0x03ul << (pin_number << 1));
	port->OSPEEDR |= (speed << (pin_number << 1));

	if(pin_number < 8)
	{
		port->AFR[0] &= ~(0x0Ful << (pin_number << 2));
		port->AFR[0] |= (alternate_function << (pin_number << 2));
	}
  else
	{
		pin_number -= 8;
		port->AFR[1] &= ~(0x0Ful << (pin_number << 2));
		port->AFR[1] |= (alternate_function << (pin_number << 2));
	}
}

/*
 * gpio_enable_pullup
 * See header file for details
 */
void gpio_enable_pullup(GPIO_TypeDef * gpio, uint8_t pin_number)
{
  gpio->PUPDR |= ((uint32_t) 1 << (2*pin_number));
}

#ifndef GPIO_FUNC_INLINE

/*
 * gpio_set
 * See header file for details
 */
void gpio_set(GPIO_TypeDef * gpio, uint8_t pin_number)
{
  gpio->BSRR |= ((uint32_t) 1 << pin_number);
}

/*
 * gpio_clr
 * See header file for details
 */
void gpio_clr(GPIO_TypeDef * gpio, uint8_t pin_number)
{
  gpio->BSRR |= ((uint32_t) (0x10000) << pin_number);
}

/*
 * gpio_get_status
 * See header file for details
 */
bool gpio_get_status(GPIO_TypeDef * gpio, uint8_t pin_number)
{
  return (gpio->IDR & (1 << pin_number));
}

#endif
