/**
 * @file main.c
 *
 * Basic Keil uVision project
 *
 * @author Tecnologix, 2017
 */
#include "stm32f4xx_hal.h"              // Keil::Device:STM32Cube HAL:Common
#include "gpio.h"
#include <stdbool.h>

/* =======================================================================
 * MACROS
 * ======================================================================= */

#define LED2_GPIO		(GPIOA) 
#define	LED2_PIN		(5)

/* =======================================================================
 * EXTERN SYMBOLS
 * ======================================================================= */

extern uint32_t uwTick;

/* =======================================================================
 * GLOBAL VARIABLES
 * ======================================================================= */

/* Just a modulo 2 counter */
uint32_t modulo_2 = 0;

/* Blink delay */
uint32_t delay = 1000;

bool dbg_cmd_out = false;

/* =======================================================================
 * PRIVATE FUNCTIONS
 * ======================================================================= */

/*
 * Initializes leds
 */
static void init_leds(void)
{
	/* Clock GPIOA peripheral */
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN;
	
	/* Set LED2 port and pin */
	gpio_set_pin(LED2_GPIO,LED2_PIN, TLX_GPIO_MODE_GPOUT, TLX_GPIO_SPEED_50MHZ, 0);
}



/* =======================================================================
 * PUBLIC FUNCTIONS
 * ======================================================================= */
 
int main(void)
{		
	HAL_Init();
	
	init_leds();
	
	gpio_set(LED2_GPIO, LED2_PIN);
	
	while(1)
	{
		modulo_2 = (modulo_2 + 1) & 1;
		
		if (modulo_2)
		{
			delay = 100;
		}
		else 
		{
			delay = 1000;
		}
		
		gpio_set(LED2_GPIO, LED2_PIN);
			
		HAL_Delay(delay);
		
		gpio_clr(LED2_GPIO, LED2_PIN);
		
		HAL_Delay(delay);
		
		/* Just a try */
		delay = 0;
		
		__nop();
		
		__nop();
	}
}


/* =======================================================================
 * INTERRUPT HANDLER
 * ======================================================================= */

/* Interrupt handler */
void SysTick_Handler(void)
{
	uwTick++;
}
